package org.dive4elements.river.artifacts;

import org.dive4elements.artifacts.CallContext;

import org.dive4elements.river.artifacts.geom.Lines;
import org.dive4elements.river.artifacts.StaticWKmsArtifact;

import org.dive4elements.river.artifacts.model.CalculationResult;
import org.dive4elements.river.artifacts.model.WKms;
import org.dive4elements.river.artifacts.model.fixings.FixRealizingResult;

import org.dive4elements.river.artifacts.states.DefaultState.ComputeType;

import org.dive4elements.river.model.FastCrossSectionLine;

import org.apache.log4j.Logger;

/**
 * The default fixation analysis artifact.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class FixationArtifact
extends      FLYSArtifact
implements   WaterLineArtifact
{
    /** The logger for this class. */
    private static Logger logger = Logger.getLogger(FixationArtifact.class);

    /** The name of the artifact. */
    public static final String ARTIFACT_NAME = "fixanalysis";

    /* FacetActivity for this artifact is registered in FixAnalysisCompute . */

    /**
     * The default constructor.
     */
    public FixationArtifact() {
        logger.debug("ctor()");
    }

    /**
     * Returns the name of the concrete artifact.
     *
     * @return the name of the concrete artifact.
     */
    @Override
    public String getName() {
        return ARTIFACT_NAME;
    }

    @Override
    public Lines.LineData getWaterLines(
        int                  facetIdx,
        FastCrossSectionLine      csl,
        double                      d,
        double                      w,
        CallContext           context
    ) {
        FixRealizingResult result = (FixRealizingResult)
            ((CalculationResult)this.compute(context, ComputeType.ADVANCE, false)).getData();

        WKms wkms = result.getWQKms()[facetIdx];

        double km = csl.getKm();

        // Find W at km.
        double wAtKm;

        wAtKm = StaticWKmsArtifact.getWAtKm(wkms, km);

        if (wAtKm == -1 || Double.isNaN(wAtKm)) {
            logger.warn("Waterlevel at km " + km + " unknown.");
            return new Lines.LineData(new double[][] {{}}, 0d, 0d);
        }

        // This should be FixRealizationResult, which can be getWQKms()ed
        return Lines.createWaterLines(csl.getPoints(), wAtKm);
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
